@@ -2,14 +2,14 @@
 
   digest.c -
 
-  $Author: shyouhei $
+  $Author$
   created at: Fri May 25 08:57:27 JST 2001
 
   Copyright (C) 1995-2001 Yukihiro Matsumoto
   Copyright (C) 2001-2006 Akinori MUSHA
 
   $RoughId: digest.c,v 1.16 2001/07/13 15:38:27 knu Exp $
-  $Id: digest.c 17775 2008-07-01 10:36:22Z shyouhei $
+  $Id$
 
 ************************************************/
 
@@ -29,6 +29,69 @@ RUBY_EXTERN void Init_digest_base(void);
  * Document-module: Digest
  *
  * This module provides a framework for message digest libraries.
+ *
+ * You may want to look at OpenSSL::Digest as it supports more algorithms.
+ *
+ * A cryptographic hash function is a procedure that takes data and returns a
+ * fixed bit string: the hash value, also known as _digest_. Hash functions
+ * are also called one-way functions, it is easy to compute a digest from
+ * a message, but it is infeasible to generate a message from a digest.
+ *
+ * == Examples
+ *
+ *   require 'digest'
+ *
+ *   # Compute a complete digest
+ *   Digest::SHA256.digest 'message'       #=> "\xABS\n\x13\xE4Y..."
+ *
+ *   sha256 = Digest::SHA256.new
+ *   sha256.digest 'message'               #=> "\xABS\n\x13\xE4Y..."
+ *
+ *   # Other encoding formats
+ *   Digest::SHA256.hexdigest 'message'    #=> "ab530a13e459..."
+ *   Digest::SHA256.base64digest 'message' #=> "q1MKE+RZFJgr..."
+ *
+ *   # Compute digest by chunks
+ *   md5 = Digest::MD5.new
+ *   md5.update 'message1'
+ *   md5 << 'message2'                     # << is an alias for update
+ *
+ *   md5.hexdigest                         #=> "94af09c09bb9..."
+ *
+ *   # Compute digest for a file
+ *   sha256 = Digest::SHA256.file 'testfile'
+ *   sha256.hexdigest
+ *
+ * Additionally digests can be encoded in "bubble babble" format as a sequence
+ * of consonants and vowels which is more recognizable and comparable than a
+ * hexadecimal digest.
+ *
+ *   require 'digest/bubblebabble'
+ *
+ *   Digest::SHA256.bubblebabble 'message' #=> "xopoh-fedac-fenyh-..."
+ *
+ * See the bubble babble specification at
+ * http://web.mit.edu/kenta/www/one/bubblebabble/spec/jrtrjwzi/draft-huima-01.txt.
+ *
+ * == Digest algorithms
+ *
+ * Different digest algorithms (or hash functions) are available:
+ *
+ * HMAC::
+ *   See FIPS PUB 198 The Keyed-Hash Message Authentication Code (HMAC).
+ * RIPEMD-160::
+ *   As Digest::RMD160.
+ *   See http://homes.esat.kuleuven.be/~bosselae/ripemd160.html.
+ * SHA1::
+ *   See FIPS 180 Secure Hash Standard.
+ * SHA2 family::
+ *   See FIPS 180 Secure Hash Standard which defines the following algorithms:
+ *   * SHA512
+ *   * SHA384
+ *   * SHA256
+ *
+ * The latest versions of the FIPS publications can be found here:
+ * http://csrc.nist.gov/publications/PubsFIPS.html.
  */
 
 static VALUE
@@ -36,7 +99,7 @@ hexencode_str_new(VALUE str_digest)
 {
     char *digest;
     size_t digest_len;
-    int i;
+    size_t i;
     VALUE str;
     char *p;
     static const char hex[] = {
@@ -52,7 +115,7 @@ hexencode_str_new(VALUE str_digest)
         rb_raise(rb_eRuntimeError, "digest string too long");
     }
 
-    str = rb_str_new(0, digest_len * 2);
+    str = rb_usascii_str_new(0, digest_len * 2);
 
     for (i = 0, p = RSTRING_PTR(str); i < digest_len; i++) {
         unsigned char byte = digest[i];
@@ -76,6 +139,8 @@ rb_digest_s_hexencode(VALUE klass, VALUE str)
     return hexencode_str_new(str);
 }
 
+NORETURN(static void rb_digest_instance_method_unimpl(VALUE self, const char *method));
+
 /*
  * Document-module: Digest::Instance
  *
@@ -83,6 +148,13 @@ rb_digest_s_hexencode(VALUE klass, VALUE str)
  * object to calculate message digest values.
  */
 
+static void
+rb_digest_instance_method_unimpl(VALUE self, const char *method)
+{
+    rb_raise(rb_eRuntimeError, "%s does not implement %s()",
+	     rb_obj_classname(self), method);
+}
+
 /*
  * call-seq:
  *     digest_obj.update(string) -> digest_obj
@@ -97,7 +169,9 @@ rb_digest_s_hexencode(VALUE klass, VALUE str)
 static VALUE
 rb_digest_instance_update(VALUE self, VALUE str)
 {
-    rb_raise(rb_eRuntimeError, "%s does not implement update()", RSTRING_PTR(rb_inspect(self)));
+    rb_digest_instance_method_unimpl(self, "update");
+
+    UNREACHABLE;
 }
 
 /*
@@ -115,7 +189,9 @@ rb_digest_instance_update(VALUE self, VALUE str)
 static VALUE
 rb_digest_instance_finish(VALUE self)
 {
-    rb_raise(rb_eRuntimeError, "%s does not implement finish()", RSTRING_PTR(rb_inspect(self)));
+    rb_digest_instance_method_unimpl(self, "finish");
+
+    UNREACHABLE;
 }
 
 /*
@@ -129,7 +205,9 @@ rb_digest_instance_finish(VALUE self)
 static VALUE
 rb_digest_instance_reset(VALUE self)
 {
-    rb_raise(rb_eRuntimeError, "%s does not implement reset()", RSTRING_PTR(rb_inspect(self)));
+    rb_digest_instance_method_unimpl(self, "reset");
+
+    UNREACHABLE;
 }
 
 /*
@@ -170,10 +248,7 @@ rb_digest_instance_digest(int argc, VALUE *argv, VALUE self)
         value = rb_funcall(self, id_finish, 0);
         rb_funcall(self, id_reset, 0);
     } else {
-        VALUE clone = rb_obj_clone(self);
-
-        value = rb_funcall(clone, id_finish, 0);
-        rb_funcall(clone, id_reset, 0);
+        value = rb_funcall(rb_obj_clone(self), id_finish, 0);
     }
 
     return value;
@@ -218,10 +293,7 @@ rb_digest_instance_hexdigest(int argc, VALUE *argv, VALUE self)
         value = rb_funcall(self, id_finish, 0);
         rb_funcall(self, id_reset, 0);
     } else {
-        VALUE clone = rb_obj_clone(self);
-
-        value = rb_funcall(clone, id_finish, 0);
-        rb_funcall(clone, id_reset, 0);
+        value = rb_funcall(rb_obj_clone(self), id_finish, 0);
     }
 
     return hexencode_str_new(value);
@@ -231,8 +303,8 @@ rb_digest_instance_hexdigest(int argc, VALUE *argv, VALUE self)
  * call-seq:
  *     digest_obj.hexdigest! -> string
  *
- * Returns the resulting hash value and resets the digest to the
- * initial state.
+ * Returns the resulting hash value in a hex-encoded form and resets
+ * the digest to the initial state.
  */
 static VALUE
 rb_digest_instance_hexdigest_bang(VALUE self)
@@ -358,7 +430,9 @@ rb_digest_instance_length(VALUE self)
 static VALUE
 rb_digest_instance_block_length(VALUE self)
 {
-    rb_raise(rb_eRuntimeError, "%s does not implement block_length()", RSTRING_PTR(rb_inspect(self)));
+    rb_digest_instance_method_unimpl(self, "block_length");
+
+    UNREACHABLE;
 }
 
 /*
@@ -412,6 +486,13 @@ rb_digest_class_s_hexdigest(int argc, VALUE *argv, VALUE klass)
     return hexencode_str_new(rb_funcall2(klass, id_digest, argc, argv));
 }
 
+/* :nodoc: */
+static VALUE
+rb_digest_class_init(VALUE self)
+{
+    return self;
+}
+
 /*
  * Document-class: Digest::Base
  *
@@ -426,14 +507,14 @@ get_digest_base_metadata(VALUE klass)
     VALUE obj;
     rb_digest_metadata_t *algo;
 
-    for (p = klass; p; p = RCLASS_SUPER(p)) {
+    for (p = klass; !NIL_P(p); p = rb_class_superclass(p)) {
         if (rb_ivar_defined(p, id_metadata)) {
             obj = rb_ivar_get(p, id_metadata);
             break;
         }
     }
 
-    if (!p)
+    if (NIL_P(p))
         rb_raise(rb_eRuntimeError, "Digest::Base cannot be directly inherited in Ruby");
 
     Data_Get_Struct(obj, rb_digest_metadata_t, algo);
@@ -619,6 +700,7 @@ Init_digest(void)
      * class Digest::Class
      */
     rb_cDigest_Class = rb_define_class_under(rb_mDigest, "Class", rb_cObject);
+    rb_define_method(rb_cDigest_Class, "initialize",  rb_digest_class_init, 0);
     rb_include_module(rb_cDigest_Class, rb_mDigest_Instance);
 
     /* class methods */
