@@ -49,7 +49,7 @@
 
 /*$OrigId: md5c.c,v 1.2 2001/03/26 08:57:14 matz Exp $ */
 /*$RoughId: md5.c,v 1.2 2001/07/13 19:48:41 knu Exp $ */
-/*$Id: md5.c 11708 2007-02-12 23:01:19Z shyouhei $ */
+/*$Id$ */
 
 #include "md5.h"
 
@@ -60,6 +60,7 @@
  * A.5 of RFC 1321, reproduced below.
  */
 #include <string.h>
+int
 main()
 {
     static const char *const test[7*2] = {
@@ -100,6 +101,7 @@ main()
  */
 #ifdef COMPUTE_T_VALUES
 #include <math.h>
+int
 main()
 {
     int i;
@@ -366,7 +368,7 @@ MD5_Update(MD5_CTX *pms, const uint8_t *data, size_t nbytes)
     size_t offset = (pms->count[0] >> 3) & 63;
     uint32_t nbits = (uint32_t)(nbytes << 3);
 
-    if (nbytes <= 0)
+    if (nbytes == 0)
 	return;
 
     /* Update the message length. */
