@@ -16,7 +16,17 @@ end
 class Tk::Iwidgets::Hyperhelp
   TkCommandNames = ['::iwidgets::hyperhelp'.freeze].freeze
   WidgetClassName = 'Hyperhelp'.freeze
-  WidgetClassNames[WidgetClassName] = self
+  WidgetClassNames[WidgetClassName] ||= self
+
+  def __strval_optkeys
+    super() << 'helpdir'
+  end
+  private :__strval_optkeys
+
+  def __listval_optkeys
+    super() << 'topics'
+  end
+  private :__listval_optkeys
 
   def show_topic(topic)
     tk_call(@path, 'showtopic', topic)
