@@ -2,8 +2,8 @@
 
   rubysig.h -
 
-  $Author: akr $
-  $Date: 2009-10-02 12:04:37 +0000 (Fri, 02 Oct 2009) $
+  $Author$
+  $Date$
   created at: Wed Aug 16 01:15:38 JST 1995
 
   Copyright (C) 1993-2008 Yukihiro Matsumoto
@@ -27,6 +27,8 @@ extern "C" {
 #endif
 #endif
 
+RUBY_SYMBOL_EXPORT_BEGIN
+
 struct rb_blocking_region_buffer;
 DEPRECATED(RUBY_EXTERN struct rb_blocking_region_buffer *rb_thread_blocking_region_begin(void));
 DEPRECATED(RUBY_EXTERN void rb_thread_blocking_region_end(struct rb_blocking_region_buffer *));
@@ -38,6 +40,8 @@ DEPRECATED(RUBY_EXTERN void rb_thread_blocking_region_end(struct rb_blocking_reg
 #define ALLOW_INTS do {CHECK_INTS;} while (0)
 #define CHECK_INTS rb_thread_check_ints()
 
+RUBY_SYMBOL_EXPORT_END
+
 #if defined(__cplusplus)
 #if 0
 { /* satisfy cc-mode */
