@@ -1,12 +1,13 @@
 #!/usr/bin/env ruby
 #
 # $RoughId: test.rb,v 1.4 2001/07/13 15:38:27 knu Exp $
-# $Id: test_digest.rb 27586 2010-05-02 05:56:35Z nobu $
+# $Id$
 
 require 'test/unit'
+require 'tempfile'
 
 require 'digest'
-%w[digest/md5 digest/rmd160 digest/sha1 digest/sha2].each do |lib|
+%w[digest/md5 digest/rmd160 digest/sha1 digest/sha2 digest/bubblebabble].each do |lib|
   begin
     require lib
   rescue LoadError
@@ -17,23 +18,35 @@ module TestDigest
   Data1 = "abc"
   Data2 = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
 
+  def test_s_new
+    self.class::DATA.each do |str, hexdigest|
+      assert_raise(ArgumentError) { self.class::ALGO.new("") }
+    end
+  end
+
   def test_s_hexdigest
     self.class::DATA.each do |str, hexdigest|
-      assert_equal(hexdigest, self.class::ALGO.hexdigest(str))
+      actual = self.class::ALGO.hexdigest(str)
+      assert_equal(hexdigest, actual)
+      assert_equal(Encoding::US_ASCII, actual.encoding)
     end
   end
 
   def test_s_base64digest
     self.class::DATA.each do |str, hexdigest|
       digest = [hexdigest].pack("H*")
-      assert_equal([digest].pack("m0"), self.class::ALGO.base64digest(str))
+      actual = self.class::ALGO.base64digest(str)
+      assert_equal([digest].pack("m0"), actual)
+      assert_equal(Encoding::US_ASCII, actual.encoding)
     end
   end
 
   def test_s_digest
     self.class::DATA.each do |str, hexdigest|
       digest = [hexdigest].pack("H*")
-      assert_equal(digest, self.class::ALGO.digest(str))
+      actual = self.class::ALGO.digest(str)
+      assert_equal(digest, actual)
+      assert_equal(Encoding::BINARY, actual.encoding)
     end
   end
 
@@ -66,12 +79,48 @@ module TestDigest
     assert_equal(md1, md2, self.class::ALGO)
   end
 
+  def test_s_file
+    Tempfile.create("test_digest_file", mode: File::BINARY) { |tmpfile|
+      str = "hello, world.\r\n"
+      tmpfile.print str
+      tmpfile.close
+
+      assert_equal self.class::ALGO.new.update(str), self.class::ALGO.file(tmpfile.path)
+    }
+  end
+
   def test_instance_eval
     assert_nothing_raised {
       self.class::ALGO.new.instance_eval { update "a" }
     }
   end
 
+  def test_alignment
+    md = self.class::ALGO.new
+    assert_nothing_raised('#4320') {
+      md.update('a' * 97)
+      md.update('a' * 97)
+      md.hexdigest
+    }
+  end
+
+  def test_bubblebabble
+    expected = "xirek-hasol-fumik-lanax"
+    assert_equal expected, Digest.bubblebabble('message')
+  end
+
+  def test_bubblebabble_class
+    expected = "xopoh-fedac-fenyh-nehon-mopel-nivor-lumiz-rypon-gyfot-cosyz-rimez-lolyv-pekyz-rosud-ricob-surac-toxox"
+    assert_equal expected, Digest::SHA256.bubblebabble('message')
+  end
+
+  def test_bubblebabble_instance
+    expected = "xumor-boceg-dakuz-sulic-gukoz-rutas-mekek-zovud-gunap-vabov-genin-rygyg-sanun-hykac-ruvah-dovah-huxex"
+
+    hash = Digest::SHA256.new
+    assert_equal expected, hash.bubblebabble
+  end
+
   class TestMD5 < Test::Unit::TestCase
     include TestDigest
     ALGO = Digest::MD5
@@ -117,6 +166,20 @@ module TestDigest
     }
   end if defined?(Digest::SHA512)
 
+  class TestSHA2 < Test::Unit::TestCase
+
+  def test_s_file
+    Tempfile.create("test_digest_file") { |tmpfile|
+      str = Data1
+      tmpfile.print str
+      tmpfile.close
+
+      assert_equal "cb00753f45a35e8bb5a03d699ac65007272c32ab0eded1631a8b605a43ff5bed8086072ba1e7cc2358baeca134c825a7", Digest::SHA2.file(tmpfile.path, 384).hexdigest
+    }
+  end
+
+  end if defined?(Digest::SHA2)
+
   class TestRMD160 < Test::Unit::TestCase
     include TestDigest
     ALGO = Digest::RMD160
@@ -125,4 +188,11 @@ module TestDigest
       Data2 => "12a053384a9c0c88e405a06c27dcf49ada62eb2b",
     }
   end if defined?(Digest::RMD160)
+
+  class TestBase < Test::Unit::TestCase
+    def test_base
+      bug3810 = '[ruby-core:32231]'
+      assert_raise(NotImplementedError, bug3810) {Digest::Base.new}
+    end
+  end
 end
