@@ -1,13 +1,29 @@
-# $Id: test_fileutils.rb,v 1.35 2005/05/26 11:33:16 aamine Exp $
+# $Id$
 
 require 'fileutils'
-require 'fileasserts'
+require_relative 'fileasserts'
 require 'pathname'
 require 'tmpdir'
 require 'test/unit'
 
 class TestFileUtils < Test::Unit::TestCase
   TMPROOT = "#{Dir.tmpdir}/fileutils.rb.#{$$}"
+  include Test::Unit::FileAssertions
+
+  def assert_output_lines(expected, fu = self, message=nil)
+    old = fu.instance_variable_get(:@fileutils_output)
+    read, write = IO.pipe
+    fu.instance_variable_set(:@fileutils_output, write)
+    th = Thread.new { read.read }
+
+    yield
+
+    write.close
+    lines = th.value.lines.map {|l| l.chomp }
+    assert_equal(expected, lines)
+  ensure
+    fu.instance_variable_set(:@fileutils_output, old) if old
+  end
 end
 
 prevdir = Dir.pwd
@@ -16,11 +32,11 @@ Dir.mkdir tmproot unless File.directory?(tmproot)
 Dir.chdir tmproot
 
 def have_drive_letter?
-  /djgpp|mswin(?!ce)|mingw|bcc|emx/ =~ RUBY_PLATFORM
+  /mswin(?!ce)|mingw|bcc|emx/ =~ RUBY_PLATFORM
 end
 
 def have_file_perm?
-  /djgpp|mswin|mingw|bcc|wince|emx/ !~ RUBY_PLATFORM
+  /mswin|mingw|bcc|emx/ !~ RUBY_PLATFORM
 end
 
 $fileutils_rb_have_symlink = nil
@@ -76,6 +92,10 @@ class TestFileUtils
 
   include FileUtils
 
+  def check_singleton(name)
+    assert_respond_to ::FileUtils, name
+  end
+
   def my_rm_rf(path)
     if File.exist?('/bin/rm')
       system %Q[/bin/rm -rf "#{path}"]
@@ -100,9 +120,8 @@ class TestFileUtils
   end
 
   def teardown
-    tmproot = Dir.pwd
     Dir.chdir @prevdir
-    my_rm_rf tmproot
+    my_rm_rf TMPROOT
   end
 
 
@@ -164,27 +183,25 @@ class TestFileUtils
   #
 
   def test_pwd
+    check_singleton :pwd
+
     assert_equal Dir.pwd, pwd()
 
     cwd = Dir.pwd
-if have_drive_letter?
-    cd('C:/') {
-      assert_equal 'C:/', pwd()
+    root = have_drive_letter? ? 'C:/' : '/'
+    cd(root) {
+      assert_equal root, pwd()
     }
     assert_equal cwd, pwd()
-else
-    cd('/') {
-      assert_equal '/', pwd()
-    }
-    assert_equal cwd, pwd()
-end
   end
 
   def test_cmp
+    check_singleton :cmp
+
     TARGETS.each do |fname|
       assert cmp(fname, fname), 'not same?'
     end
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       cmp TARGETS[0], TARGETS[0], :undefinedoption => true
     }
 
@@ -198,6 +215,8 @@ end
   end
 
   def test_cp
+    check_singleton :cp
+
     each_srcdest do |srcpath, destpath|
       cp srcpath, destpath
       assert_same_file srcpath, destpath
@@ -213,28 +232,58 @@ end
       assert_same_entry srcpath, destpath
     end
 
+    assert_raise(Errno::ENOENT) {
+      cp 'tmp/cptmp', 'tmp/cptmp_new'
+    }
+    assert_file_not_exist('tmp/cptmp_new')
+
     # src==dest (1) same path
     touch 'tmp/cptmp'
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       cp 'tmp/cptmp', 'tmp/cptmp'
     }
-if have_symlink?
+  end
+
+  def test_cp_preserve_permissions
+    bug4507 = '[ruby-core:35518]'
+    touch 'tmp/cptmp'
+    chmod 0755, 'tmp/cptmp'
+    cp 'tmp/cptmp', 'tmp/cptmp2'
+    assert_equal_filemode('tmp/cptmp', 'tmp/cptmp2', bug4507)
+  end
+
+  def test_cp_preserve_permissions_dir
+    bug7246 = '[ruby-core:48603]'
+    mkdir 'tmp/cptmp'
+    mkdir 'tmp/cptmp/d1'
+    chmod 0745, 'tmp/cptmp/d1'
+    mkdir 'tmp/cptmp/d2'
+    chmod 0700, 'tmp/cptmp/d2'
+    cp_r 'tmp/cptmp', 'tmp/cptmp2', :preserve => true
+    assert_equal_filemode('tmp/cptmp/d1', 'tmp/cptmp2/d1', bug7246)
+    assert_equal_filemode('tmp/cptmp/d2', 'tmp/cptmp2/d2', bug7246)
+  end
+
+  def test_cp_symlink
+    touch 'tmp/cptmp'
     # src==dest (2) symlink and its target
     File.symlink 'cptmp', 'tmp/cptmp_symlink'
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       cp 'tmp/cptmp', 'tmp/cptmp_symlink'
     }
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       cp 'tmp/cptmp_symlink', 'tmp/cptmp'
     }
     # src==dest (3) looped symlink
     File.symlink 'symlink', 'tmp/symlink'
-    assert_raises(Errno::ELOOP) {
+    assert_raise(Errno::ELOOP) {
       cp 'tmp/symlink', 'tmp/symlink'
     }
-end
+  end if have_symlink?
 
+  def test_cp_pathname
     # pathname
+    touch 'tmp/cptmp'
     assert_nothing_raised {
       cp 'tmp/cptmp', Pathname.new('tmp/tmpdest')
       cp Pathname.new('tmp/cptmp'), 'tmp/tmpdest'
@@ -245,6 +294,8 @@ end
   end
 
   def test_cp_r
+    check_singleton :cp_r
+
     cp_r 'data', 'tmp'
     TARGETS.each do |fname|
       assert_same_file fname, "tmp/#{fname}"
@@ -271,7 +322,17 @@ end
     my_rm_rf 'tmp/cpr_src'
     my_rm_rf 'tmp/cpr_dest'
 
-if have_symlink?
+    bug3588 = '[ruby-core:31360]'
+    assert_nothing_raised(ArgumentError, bug3588) do
+      cp_r 'tmp', 'tmp2'
+    end
+    assert_directory 'tmp2/tmp'
+    assert_raise(ArgumentError, bug3588) do
+      cp_r 'tmp2', 'tmp2/new_tmp2'
+    end
+  end
+
+  def test_cp_r_symlink
     # symlink in a directory
     mkdir 'tmp/cpr_src'
     ln_s 'SLdest', 'tmp/cpr_src/symlink'
@@ -283,12 +344,25 @@ if have_symlink?
     ln_s 'cpr_src', 'tmp/cpr_src2'
     cp_r 'tmp/cpr_src2', 'tmp/cpr_dest2'
     assert_directory 'tmp/cpr_dest2'
-    #assert_not_symlink 'tmp/cpr_dest2'
-    assert_symlink 'tmp/cpr_dest2'   # 2005-05-26: feature change
+    assert_not_symlink 'tmp/cpr_dest2'
     assert_symlink 'tmp/cpr_dest2/symlink'
     assert_equal 'SLdest', File.readlink('tmp/cpr_dest2/symlink')
-end
+  end if have_symlink?
 
+  def test_cp_r_symlink_preserve
+    mkdir 'tmp/cross'
+    mkdir 'tmp/cross/a'
+    mkdir 'tmp/cross/b'
+    touch 'tmp/cross/a/f'
+    touch 'tmp/cross/b/f'
+    ln_s '../a/f', 'tmp/cross/b/l'
+    ln_s '../b/f', 'tmp/cross/a/l'
+    assert_nothing_raised {
+      cp_r 'tmp/cross', 'tmp/cross2', :preserve => true
+    }
+  end if have_symlink?
+
+  def test_cp_r_pathname
     # pathname
     touch 'tmp/cprtmp'
     assert_nothing_raised {
@@ -299,6 +373,8 @@ end
   end
 
   def test_mv
+    check_singleton :mv
+
     mkdir 'tmp/dest'
     TARGETS.each do |fname|
       cp fname, 'tmp/mvsrc'
@@ -312,38 +388,41 @@ end
       assert_same_file fname, 'tmp/mvdest'
     end
 
-    # [ruby-talk:124368]
     mkdir 'tmp/tmpdir'
     mkdir_p 'tmp/dest2/tmpdir'
-    assert_raises(Errno::EEXIST) {
+    assert_raise(Errno::EEXIST) {
       mv 'tmp/tmpdir', 'tmp/dest2'
     }
     mkdir 'tmp/dest2/tmpdir/junk'
-    assert_raises(Errno::EEXIST) {
+    assert_raise(Errno::EEXIST, "[ruby-talk:124368]") {
       mv 'tmp/tmpdir', 'tmp/dest2'
     }
 
     # src==dest (1) same path
     touch 'tmp/cptmp'
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       mv 'tmp/cptmp', 'tmp/cptmp'
     }
-if have_symlink?
+  end
+
+  def test_mv_symlink
+    touch 'tmp/cptmp'
     # src==dest (2) symlink and its target
     File.symlink 'cptmp', 'tmp/cptmp_symlink'
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       mv 'tmp/cptmp', 'tmp/cptmp_symlink'
     }
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       mv 'tmp/cptmp_symlink', 'tmp/cptmp'
     }
     # src==dest (3) looped symlink
     File.symlink 'symlink', 'tmp/symlink'
-    assert_raises(Errno::ELOOP) {
+    assert_raise(Errno::ELOOP) {
       mv 'tmp/symlink', 'tmp/symlink'
     }
-end
+  end if have_symlink?
 
+  def test_mv_pathname
     # pathname
     assert_nothing_raised {
       touch 'tmp/mvtmpsrc'
@@ -356,6 +435,8 @@ end
   end
 
   def test_rm
+    check_singleton :rm
+
     TARGETS.each do |fname|
       cp fname, 'tmp/rmsrc'
       rm 'tmp/rmsrc'
@@ -376,19 +457,21 @@ end
   end
 
   def test_rm_f
+    check_singleton :rm_f
+
     TARGETS.each do |fname|
       cp fname, 'tmp/rmsrc'
       rm_f 'tmp/rmsrc'
       assert_file_not_exist 'tmp/rmsrc'
     end
+  end
 
-if have_symlink?
+  def test_rm_symlink
     File.open('tmp/lnf_symlink_src', 'w') {|f| f.puts 'dummy' }
     File.symlink 'tmp/lnf_symlink_src', 'tmp/lnf_symlink_dest'
     rm_f 'tmp/lnf_symlink_dest'
     assert_file_not_exist 'tmp/lnf_symlink_dest'
     assert_file_exist     'tmp/lnf_symlink_src'
-end
 
     rm_f 'notexistdatafile'
     rm_f 'tmp/notexistdatafile'
@@ -396,7 +479,9 @@ end
     Dir.mkdir 'tmpdatadir'
     # rm_f 'tmpdatadir'
     Dir.rmdir 'tmpdatadir'
+  end if have_symlink?
 
+  def test_rm_f_2
     Dir.mkdir 'tmp/tmpdir'
     File.open('tmp/tmpdir/a', 'w') {|f| f.puts 'dummy' }
     File.open('tmp/tmpdir/c', 'w') {|f| f.puts 'dummy' }
@@ -404,7 +489,9 @@ end
     assert_file_not_exist 'tmp/tmpdir/a'
     assert_file_not_exist 'tmp/tmpdir/c'
     Dir.rmdir 'tmp/tmpdir'
+  end
 
+  def test_rm_pathname
     # pathname
     touch 'tmp/rmtmp1'
     touch 'tmp/rmtmp2'
@@ -418,9 +505,16 @@ end
     assert_file_not_exist 'tmp/rmtmp2'
     assert_file_not_exist 'tmp/rmtmp3'
     assert_file_exist 'tmp/rmtmp4'
+
+    # [ruby-dev:39345]
+    touch 'tmp/[rmtmp]'
+    FileUtils.rm_f 'tmp/[rmtmp]'
+    assert_file_not_exist 'tmp/[rmtmp]'
   end
 
   def test_rm_r
+    check_singleton :rm_r
+
     my_rm_rf 'tmpdatadir'
 
     Dir.mkdir 'tmpdatadir'
@@ -456,16 +550,18 @@ end
     assert_file_not_exist 'tmp/tmpdir/a'
     assert_file_not_exist 'tmp/tmpdir/c'
     Dir.rmdir 'tmp/tmpdir'
+  end
 
-if have_symlink?
+  def test_rm_r_symlink
     # [ruby-talk:94635] a symlink to the directory
     Dir.mkdir 'tmp/tmpdir'
     File.symlink '..', 'tmp/tmpdir/symlink_to_dir'
     rm_r 'tmp/tmpdir'
     assert_file_not_exist 'tmp/tmpdir'
     assert_file_exist     'tmp'
-end
+  end if have_symlink?
 
+  def test_rm_r_pathname
     # pathname
     Dir.mkdir 'tmp/tmpdir1'; touch 'tmp/tmpdir1/tmp'
     Dir.mkdir 'tmp/tmpdir2'; touch 'tmp/tmpdir2/tmp'
@@ -479,6 +575,66 @@ end
     assert_file_not_exist 'tmp/tmpdir3'
   end
 
+  def test_remove_entry_secure
+    check_singleton :remove_entry_secure
+
+    my_rm_rf 'tmpdatadir'
+
+    Dir.mkdir 'tmpdatadir'
+    remove_entry_secure 'tmpdatadir'
+    assert_file_not_exist 'tmpdatadir'
+
+    Dir.mkdir 'tmpdatadir'
+    remove_entry_secure 'tmpdatadir/'
+    assert_file_not_exist 'tmpdatadir'
+
+    Dir.mkdir 'tmp/tmpdir'
+    remove_entry_secure 'tmp/tmpdir/'
+    assert_file_not_exist 'tmp/tmpdir'
+    assert_file_exist     'tmp'
+
+    Dir.mkdir 'tmp/tmpdir'
+    remove_entry_secure 'tmp/tmpdir'
+    assert_file_not_exist 'tmp/tmpdir'
+    assert_file_exist     'tmp'
+
+    Dir.mkdir 'tmp/tmpdir'
+    File.open('tmp/tmpdir/a', 'w') {|f| f.puts 'dummy' }
+    File.open('tmp/tmpdir/b', 'w') {|f| f.puts 'dummy' }
+    File.open('tmp/tmpdir/c', 'w') {|f| f.puts 'dummy' }
+    remove_entry_secure 'tmp/tmpdir'
+    assert_file_not_exist 'tmp/tmpdir'
+    assert_file_exist     'tmp'
+
+    Dir.mkdir 'tmp/tmpdir'
+    File.open('tmp/tmpdir/a', 'w') {|f| f.puts 'dummy' }
+    File.open('tmp/tmpdir/c', 'w') {|f| f.puts 'dummy' }
+    remove_entry_secure 'tmp/tmpdir/a', true
+    remove_entry_secure 'tmp/tmpdir/b', true
+    remove_entry_secure 'tmp/tmpdir/c', true
+    assert_file_not_exist 'tmp/tmpdir/a'
+    assert_file_not_exist 'tmp/tmpdir/c'
+    Dir.rmdir 'tmp/tmpdir'
+  end
+
+  def test_remove_entry_secure_symlink
+    # [ruby-talk:94635] a symlink to the directory
+    Dir.mkdir 'tmp/tmpdir'
+    File.symlink '..', 'tmp/tmpdir/symlink_to_dir'
+    remove_entry_secure 'tmp/tmpdir'
+    assert_file_not_exist 'tmp/tmpdir'
+    assert_file_exist     'tmp'
+  end if have_symlink?
+
+  def test_remove_entry_secure_pathname
+    # pathname
+    Dir.mkdir 'tmp/tmpdir1'; touch 'tmp/tmpdir1/tmp'
+    assert_nothing_raised {
+      remove_entry_secure Pathname.new('tmp/tmpdir1')
+    }
+    assert_file_not_exist 'tmp/tmpdir1'
+  end
+
   def test_with_big_file
     prepare_big_file
 
@@ -494,7 +650,6 @@ end
     assert_file_not_exist 'tmp/mvdest'
   end
 
-if have_hardlink?
   def test_ln
     TARGETS.each do |fname|
       ln fname, 'tmp/lndest'
@@ -512,16 +667,19 @@ if have_hardlink?
 
     # src==dest (1) same path
     touch 'tmp/cptmp'
-    assert_raises(Errno::EEXIST) {
+    assert_raise(Errno::EEXIST) {
       ln 'tmp/cptmp', 'tmp/cptmp'
     }
-if have_symlink?
+  end if have_hardlink?
+
+  def test_ln_symlink
+    touch 'tmp/cptmp'
     # src==dest (2) symlink and its target
     File.symlink 'cptmp', 'tmp/symlink'
-    assert_raises(Errno::EEXIST) {
+    assert_raise(Errno::EEXIST) {
       ln 'tmp/cptmp', 'tmp/symlink'   # normal file -> symlink
     }
-    assert_raises(Errno::EEXIST) {
+    assert_raise(Errno::EEXIST) {
       ln 'tmp/symlink', 'tmp/cptmp'   # symlink -> normal file
     }
     # src==dest (3) looped symlink
@@ -531,8 +689,9 @@ if have_symlink?
     rescue => err
       assert_kind_of SystemCallError, err
     end
-end
+  end if have_symlink?
 
+  def test_ln_pathname
     # pathname
     touch 'tmp/lntmp'
     assert_nothing_raised {
@@ -540,11 +699,11 @@ end
       ln 'tmp/lntmp', Pathname.new('tmp/lndesttmp2')
       ln Pathname.new('tmp/lntmp'), Pathname.new('tmp/lndesttmp3')
     }
-  end
-end
+  end if have_hardlink?
 
-if have_symlink?
   def test_ln_s
+    check_singleton :ln_s
+
     TARGETS.each do |fname|
       ln_s fname, 'tmp/lnsdest'
       assert FileTest.symlink?('tmp/lnsdest'), 'not symlink'
@@ -563,11 +722,11 @@ if have_symlink?
       ln_s 'lnsdest', Pathname.new('tmp/symlink_tmp2')
       ln_s Pathname.new('lnsdest'), Pathname.new('tmp/symlink_tmp3')
     }
-  end
-end
+  end if have_symlink?
 
-if have_symlink?
   def test_ln_sf
+    check_singleton :ln_sf
+
     TARGETS.each do |fname|
       ln_sf fname, 'tmp/lnsdest'
       assert FileTest.symlink?('tmp/lnsdest'), 'not symlink'
@@ -586,10 +745,11 @@ if have_symlink?
       ln_sf 'lns_dest', Pathname.new('tmp/symlink_tmp2')
       ln_sf Pathname.new('lns_dest'), Pathname.new('tmp/symlink_tmp3')
     }
-  end
-end
+  end if have_symlink?
 
   def test_mkdir
+    check_singleton :mkdir
+
     my_rm_rf 'tmpdatadir'
     mkdir 'tmpdatadir'
     assert_directory 'tmpdatadir'
@@ -605,22 +765,29 @@ end
 
     mkdir 'tmp/tmp', :mode => 0700
     assert_directory 'tmp/tmp'
-    assert_equal 0700, (File.stat('tmp/tmp').mode & 0777) if have_file_perm?
+    assert_filemode 0700, 'tmp/tmp', mask: 0777 if have_file_perm?
     Dir.rmdir 'tmp/tmp'
 
-if have_file_perm?
+    # EISDIR on OS X, FreeBSD; EEXIST on Linux; Errno::EACCES on Windows
+    assert_raise(Errno::EISDIR, Errno::EEXIST, Errno::EACCES) {
+      mkdir '/'
+    }
+  end
+
+  def test_mkdir_file_perm
     mkdir 'tmp/tmp', :mode => 07777
     assert_directory 'tmp/tmp'
-    assert_equal 07777, (File.stat('tmp/tmp').mode & 07777)
+    assert_filemode 07777, 'tmp/tmp'
     Dir.rmdir 'tmp/tmp'
-end
+  end if have_file_perm?
 
-if lf_in_path_allowed?
+  def test_mkdir_lf_in_path
     mkdir "tmp-first-line\ntmp-second-line"
     assert_directory "tmp-first-line\ntmp-second-line"
     Dir.rmdir "tmp-first-line\ntmp-second-line"
-end
+  end if lf_in_path_allowed?
 
+  def test_mkdir_pathname
     # pathname
     assert_nothing_raised {
       mkdir Pathname.new('tmp/tmpdirtmp')
@@ -629,6 +796,8 @@ end
   end
 
   def test_mkdir_p
+    check_singleton :mkdir_p
+
     dirs = %w(
       tmpdir/dir/
       tmpdir/dir/./
@@ -665,26 +834,30 @@ end
     mkdir_p 'tmp/tmp/tmp', :mode => 0700
     assert_directory 'tmp/tmp'
     assert_directory 'tmp/tmp/tmp'
-    assert_equal 0700, (File.stat('tmp/tmp').mode & 0777) if have_file_perm?
-    assert_equal 0700, (File.stat('tmp/tmp/tmp').mode & 0777) if have_file_perm?
+    assert_filemode 0700, 'tmp/tmp', mask: 0777 if have_file_perm?
+    assert_filemode 0700, 'tmp/tmp/tmp', mask: 0777 if have_file_perm?
     rm_rf 'tmp/tmp'
 
     mkdir_p 'tmp/tmp', :mode => 0
     assert_directory 'tmp/tmp'
-    assert_equal 0, (File.stat('tmp/tmp').mode & 0777) if have_file_perm?
+    assert_filemode 0, 'tmp/tmp', mask: 0777 if have_file_perm?
     # DO NOT USE rm_rf here.
     # (rm(1) try to chdir to parent directory, it fails to remove directory.)
     Dir.rmdir 'tmp/tmp'
     Dir.rmdir 'tmp'
 
-if have_file_perm?
+    mkdir_p '/'
+  end
+
+  def test_mkdir_p_file_perm
     mkdir_p 'tmp/tmp/tmp', :mode => 07777
     assert_directory 'tmp/tmp/tmp'
-    assert_equal 07777, (File.stat('tmp/tmp/tmp').mode & 07777)
+    assert_filemode 07777, 'tmp/tmp/tmp'
     Dir.rmdir 'tmp/tmp/tmp'
     Dir.rmdir 'tmp/tmp'
-end
+  end if have_file_perm?
 
+  def test_mkdir_p_pathname
     # pathname
     assert_nothing_raised {
       mkdir_p Pathname.new('tmp/tmp/tmp')
@@ -692,43 +865,49 @@ end
   end
 
   def test_install
+    check_singleton :install
+
     File.open('tmp/aaa', 'w') {|f| f.puts 'aaa' }
     File.open('tmp/bbb', 'w') {|f| f.puts 'bbb' }
     install 'tmp/aaa', 'tmp/bbb', :mode => 0600
     assert_equal "aaa\n", File.read('tmp/bbb')
-    assert_equal 0600, (File.stat('tmp/bbb').mode & 0777) if have_file_perm?
+    assert_filemode 0600, 'tmp/bbb', mask: 0777 if have_file_perm?
 
     t = File.mtime('tmp/bbb')
     install 'tmp/aaa', 'tmp/bbb'
     assert_equal "aaa\n", File.read('tmp/bbb')
-    assert_equal 0600, (File.stat('tmp/bbb').mode & 0777) if have_file_perm?
-    assert_equal t, File.mtime('tmp/bbb')
+    assert_filemode 0600, 'tmp/bbb', mask: 0777 if have_file_perm?
+    assert_equal_time t, File.mtime('tmp/bbb')
 
     File.unlink 'tmp/aaa'
     File.unlink 'tmp/bbb'
 
     # src==dest (1) same path
     touch 'tmp/cptmp'
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       install 'tmp/cptmp', 'tmp/cptmp'
     }
-if have_symlink?
+  end
+
+  def test_install_symlink
+    touch 'tmp/cptmp'
     # src==dest (2) symlink and its target
     File.symlink 'cptmp', 'tmp/cptmp_symlink'
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       install 'tmp/cptmp', 'tmp/cptmp_symlink'
     }
-    assert_raises(ArgumentError) {
+    assert_raise(ArgumentError) {
       install 'tmp/cptmp_symlink', 'tmp/cptmp'
     }
     # src==dest (3) looped symlink
     File.symlink 'symlink', 'tmp/symlink'
-    assert_raises(Errno::ELOOP) {
+    assert_raise(Errno::ELOOP) {
       # File#install invokes open(2), always ELOOP must be raised
       install 'tmp/symlink', 'tmp/symlink'
     }
-end
+  end if have_symlink?
 
+  def test_install_pathname
     # pathname
     assert_nothing_raised {
       rm_f 'tmp/a'; touch 'tmp/a'
@@ -748,47 +927,152 @@ end
     }
   end
 
-if have_file_perm?
   def test_chmod
+    check_singleton :chmod
+
     touch 'tmp/a'
     chmod 0700, 'tmp/a'
-    assert_equal 0700, File.stat('tmp/a').mode & 0777
+    assert_filemode 0700, 'tmp/a'
     chmod 0500, 'tmp/a'
-    assert_equal 0500, File.stat('tmp/a').mode & 0777
-  end
+    assert_filemode 0500, 'tmp/a'
+  end if have_file_perm?
+
+  def test_chmod_symbol_mode
+    check_singleton :chmod
+
+    touch 'tmp/a'
+    chmod "u=wrx,g=rx,o=x", 'tmp/a'
+    assert_filemode 0751, 'tmp/a'
+    chmod "g+w-x", 'tmp/a'
+    assert_filemode 0761, 'tmp/a'
+    chmod "o+r,g=o+w,o-r,u-o", 'tmp/a' # 761 => 763 => 773 => 771 => 671
+    assert_filemode 0671, 'tmp/a'
+    chmod "go=u", 'tmp/a'
+    assert_filemode 0666, 'tmp/a'
+    chmod "u=wrx,g=,o=", 'tmp/a'
+    assert_filemode 0700, 'tmp/a'
+    chmod "u=rx,go=", 'tmp/a'
+    assert_filemode 0500, 'tmp/a'
+    chmod "+wrx", 'tmp/a'
+    assert_filemode 0777, 'tmp/a'
+    chmod "u+s,o=s", 'tmp/a'
+    assert_filemode 04770, 'tmp/a'
+    chmod "u-w,go-wrx", 'tmp/a'
+    assert_filemode 04500, 'tmp/a'
+    chmod "+s", 'tmp/a'
+    assert_filemode 06500, 'tmp/a'
+
+    # FreeBSD ufs and tmpfs don't allow to change sticky bit against
+    # regular file. It's slightly strange. Anyway it's no effect bit.
+    # see /usr/src/sys/ufs/ufs/ufs_chmod()
+    # NetBSD, OpenBSD and Solaris also denies it.
+    if /freebsd|netbsd|openbsd|solaris/ !~ RUBY_PLATFORM
+      chmod "u+t,o+t", 'tmp/a'
+      assert_filemode 07500, 'tmp/a'
+      chmod "a-t,a-s", 'tmp/a'
+      assert_filemode 0500, 'tmp/a'
+    end
+
+    assert_raise_with_message(ArgumentError, /invalid\b.*\bfile mode/) {
+      chmod "a", 'tmp/a'
+    }
+
+    assert_raise_with_message(ArgumentError, /invalid\b.*\bfile mode/) {
+      chmod "x+a", 'tmp/a'
+    }
+
+    assert_raise_with_message(ArgumentError, /invalid\b.*\bfile mode/) {
+      chmod "u+z", 'tmp/a'
+    }
+
+    assert_raise_with_message(ArgumentError, /invalid\b.*\bfile mode/) {
+      chmod ",+x", 'tmp/a'
+    }
+
+    assert_raise_with_message(ArgumentError, /invalid\b.*\bfile mode/) {
+      chmod "755", 'tmp/a'
+    }
+
+  end if have_file_perm?
+
 
   def test_chmod_R
+    check_singleton :chmod_R
+
     mkdir_p 'tmp/dir/dir'
     touch %w( tmp/dir/file tmp/dir/dir/file )
     chmod_R 0700, 'tmp/dir'
-    assert_equal 0700, File.stat('tmp/dir').mode & 0777
-    assert_equal 0700, File.stat('tmp/dir/file').mode & 0777
-    assert_equal 0700, File.stat('tmp/dir/dir').mode & 0777
-    assert_equal 0700, File.stat('tmp/dir/dir/file').mode & 0777
+    assert_filemode 0700, 'tmp/dir', mask: 0777
+    assert_filemode 0700, 'tmp/dir/file', mask: 0777
+    assert_filemode 0700, 'tmp/dir/dir', mask: 0777
+    assert_filemode 0700, 'tmp/dir/dir/file', mask: 0777
     chmod_R 0500, 'tmp/dir'
-    assert_equal 0500, File.stat('tmp/dir').mode & 0777
-    assert_equal 0500, File.stat('tmp/dir/file').mode & 0777
-    assert_equal 0500, File.stat('tmp/dir/dir').mode & 0777
-    assert_equal 0500, File.stat('tmp/dir/dir/file').mode & 0777
+    assert_filemode 0500, 'tmp/dir', mask: 0777
+    assert_filemode 0500, 'tmp/dir/file', mask: 0777
+    assert_filemode 0500, 'tmp/dir/dir', mask: 0777
+    assert_filemode 0500, 'tmp/dir/dir/file', mask: 0777
     chmod_R 0700, 'tmp/dir'   # to remove
-  end
+  end if have_file_perm?
+
+  def test_chmod_symbol_mode_R
+    check_singleton :chmod_R
+
+    mkdir_p 'tmp/dir/dir'
+    touch %w( tmp/dir/file tmp/dir/dir/file )
+    chmod_R "u=wrx,g=,o=", 'tmp/dir'
+    assert_filemode 0700, 'tmp/dir', mask: 0777
+    assert_filemode 0700, 'tmp/dir/file', mask: 0777
+    assert_filemode 0700, 'tmp/dir/dir', mask: 0777
+    assert_filemode 0700, 'tmp/dir/dir/file', mask: 0777
+    chmod_R "u=xr,g+X,o=", 'tmp/dir'
+    assert_filemode 0510, 'tmp/dir', mask: 0777
+    assert_filemode 0500, 'tmp/dir/file', mask: 0777
+    assert_filemode 0510, 'tmp/dir/dir', mask: 0777
+    assert_filemode 0500, 'tmp/dir/dir/file', mask: 0777
+    chmod_R 0700, 'tmp/dir'   # to remove
+  end if have_file_perm?
+
+  def test_chmod_verbose
+    check_singleton :chmod
+
+    assert_output_lines(["chmod 700 tmp/a", "chmod 500 tmp/a"]) {
+      touch 'tmp/a'
+      chmod 0700, 'tmp/a', verbose: true
+      assert_filemode 0700, 'tmp/a', mask: 0777
+      chmod 0500, 'tmp/a', verbose: true
+      assert_filemode 0500, 'tmp/a', mask: 0777
+    }
+  end if have_file_perm?
+
+  def test_s_chmod_verbose
+    assert_output_lines(["chmod 700 tmp/a"], FileUtils) {
+      touch 'tmp/a'
+      FileUtils.chmod 0700, 'tmp/a', verbose: true
+      assert_filemode 0700, 'tmp/a', mask: 0777
+    }
+  end if have_file_perm?
 
   # FIXME: How can I test this method?
   def test_chown
-  end
+    check_singleton :chown
+  end if have_file_perm?
 
   # FIXME: How can I test this method?
   def test_chown_R
-  end
-end
+    check_singleton :chown_R
+  end if have_file_perm?
 
   def test_copy_entry
+    check_singleton :copy_entry
+
     each_srcdest do |srcpath, destpath|
       copy_entry srcpath, destpath
       assert_same_file srcpath, destpath
       assert_equal File.stat(srcpath).ftype, File.stat(destpath).ftype
     end
-if have_symlink?
+  end
+
+  def test_copy_entry_symlink
     # root is a symlink
     File.symlink 'somewhere', 'tmp/symsrc'
     copy_entry 'tmp/symsrc', 'tmp/symdest'
@@ -803,10 +1087,11 @@ if have_symlink?
     assert_not_symlink 'tmp/dirdest'
     assert_symlink 'tmp/dirdest/sym'
     assert_equal 'somewhere', File.readlink('tmp/dirdest/sym')
-end
-  end
+  end if have_symlink?
 
   def test_copy_file
+    check_singleton :copy_file
+
     each_srcdest do |srcpath, destpath|
       copy_file srcpath, destpath
       assert_same_file srcpath, destpath
@@ -814,22 +1099,24 @@ end
   end
 
   def test_copy_stream
+    check_singleton :copy_stream
     # IO
     each_srcdest do |srcpath, destpath|
-      File.open(srcpath) {|src|
-        File.open(destpath, 'w') {|dest|
+      File.open(srcpath, 'rb') {|src|
+        File.open(destpath, 'wb') {|dest|
           copy_stream src, dest
         }
       }
       assert_same_file srcpath, destpath
     end
+  end
 
+  def test_copy_stream_duck
+    check_singleton :copy_stream
     # duck typing test  [ruby-dev:25369]
-    my_rm_rf 'tmp'
-    Dir.mkdir 'tmp'
     each_srcdest do |srcpath, destpath|
-      File.open(srcpath) {|src|
-        File.open(destpath, 'w') {|dest|
+      File.open(srcpath, 'rb') {|src|
+        File.open(destpath, 'wb') {|dest|
           copy_stream Stream.new(src), Stream.new(dest)
         }
       }
@@ -838,35 +1125,41 @@ end
   end
 
   def test_remove_file
+    check_singleton :remove_file
     File.open('data/tmp', 'w') {|f| f.puts 'dummy' }
     remove_file 'data/tmp'
     assert_file_not_exist 'data/tmp'
-if have_file_perm?
+  end
+
+  def test_remove_file_file_perm
     File.open('data/tmp', 'w') {|f| f.puts 'dummy' }
     File.chmod 0, 'data/tmp'
     remove_file 'data/tmp'
     assert_file_not_exist 'data/tmp'
-end
-  end
+  end if have_file_perm?
 
   def test_remove_dir
+    check_singleton :remove_dir
     Dir.mkdir 'data/tmpdir'
     File.open('data/tmpdir/a', 'w') {|f| f.puts 'dummy' }
     remove_dir 'data/tmpdir'
     assert_file_not_exist 'data/tmpdir'
-if have_file_perm?
+  end
+
+  def test_remove_dir_file_perm
     Dir.mkdir 'data/tmpdir'
     File.chmod 0555, 'data/tmpdir'
     remove_dir 'data/tmpdir'
     assert_file_not_exist 'data/tmpdir'
-end
-  end
+  end if have_file_perm?
 
   def test_compare_file
+    check_singleton :compare_file
     # FIXME
   end
 
   def test_compare_stream
+    check_singleton :compare_stream
     # FIXME
   end
 
@@ -875,8 +1168,8 @@ end
       @f = f
     end
 
-    def read(n)
-      @f.read(n)
+    def read(*args)
+      @f.read(*args)
     end
 
     def write(str)
@@ -885,6 +1178,7 @@ end
   end
 
   def test_uptodate?
+    check_singleton :uptodate?
     prepare_time_data
     Dir.chdir('data') {
       assert(   uptodate?('newest', %w(old newer notexist)) )
@@ -902,6 +1196,102 @@ end
       uptodate? 'tmp/a', ['tmp/b', Pathname.new('tmp/c')]
       uptodate? Pathname.new('tmp/a'), [Pathname.new('tmp/b'), Pathname.new('tmp/c')]
     }
+    # [Bug #6708] [ruby-core:46256]
+    assert_raise_with_message(ArgumentError, "wrong number of arguments (3 for 2)") {
+      uptodate?('new',['old', 'oldest'], {})
+    }
+  end
+
+  def test_cd
+    check_singleton :cd
+  end
+
+  def test_chdir
+    check_singleton :chdir
+  end
+
+  def test_getwd
+    check_singleton :getwd
+  end
+
+  def test_identical?
+    check_singleton :identical?
+  end
+
+  def test_link
+    check_singleton :link
+  end
+
+  def test_makedirs
+    check_singleton :makedirs
+  end
+
+  def test_mkpath
+    check_singleton :mkpath
+  end
+
+  def test_move
+    check_singleton :move
+  end
+
+  def test_rm_rf
+    check_singleton :rm_rf
+  end
+
+  def test_rmdir
+    check_singleton :rmdir
+
+    begin
+      Dir.rmdir '/'
+    rescue Errno::ENOTEMPTY
+    rescue => e
+      assert_raise(e.class) {
+        # Dir.rmdir('') raises Errno::ENOENT.
+        # FileUtils#rmdir ignores it.
+        # And this test failed as expected.
+        rmdir '/'
+      }
+    end
+
+    subdir = 'data/sub/dir'
+    mkdir_p(subdir)
+    assert_nothing_raised(Errno::ENOENT) {
+      rmdir(subdir, parents: true)
+    }
+    assert_file_not_exist(subdir)
+    assert_file_not_exist('data/sub')
+    assert_directory('data')
+  end
+
+  def test_rmtree
+    check_singleton :rmtree
+  end
+
+  def test_safe_unlink
+    check_singleton :safe_unlink
+  end
+
+  def test_symlink
+    check_singleton :symlink
+  end
+
+  def test_touch
+    check_singleton :touch
+  end
+
+  def test_collect_methods
+  end
+
+  def test_commands
+  end
+
+  def test_have_option?
+  end
+
+  def test_options
+  end
+
+  def test_options_of
   end
 
 end
