@@ -1,8 +1,9 @@
-# -*- mode: ruby; ruby-indent-level: 4; tab-width: 4 -*-
+# -*- coding: us-ascii; mode: ruby; ruby-indent-level: 4; tab-width: 4 -*-
 #												vim:sw=4:ts=4
 # $Id$
 #
 require_relative 'helper'
+require 'ostruct'
 
 # [ruby-core:01946]
 module Psych_Tests
@@ -13,6 +14,23 @@ class Psych_Unit_Tests < Psych::TestCase
     def teardown
         Psych.domain_types.clear
     end
+
+    def test_y_method
+      assert_raises(NoMethodError) do
+        OpenStruct.new.y 1
+      end
+    end
+
+    def test_syck_compat
+      time = Time.utc(2010, 10, 10)
+      yaml = Psych.dump time
+      assert_match "2010-10-10 00:00:00.000000000 Z", yaml
+    end
+
+    # [ruby-core:34969]
+    def test_regexp_with_n
+        assert_cycle(Regexp.new('',0,'n'))
+    end
 	#
 	# Tests modified from 00basic.t in Psych.pm
 	#
@@ -1248,4 +1266,24 @@ EOY
       Psych.load("2000-01-01 00:00:00.#{"0"*1000} +00:00\n")
       # '[ruby-core:13735]'
     end
+
+    def test_multiline_string_uses_literal_style
+      yaml = Psych.dump("multi\nline\nstring")
+      assert_match("|", yaml)
+    end
+
+    def test_string_starting_with_non_word_character_uses_double_quotes_without_exclamation_mark
+      yaml = Psych.dump("@123'abc")
+      refute_match("!", yaml)
+    end
+
+    def test_string_dump_with_colon
+      yaml = Psych.dump 'x: foo'
+      refute_match '!', yaml
+    end
+
+    def test_string_dump_starting_with_star
+      yaml = Psych.dump '*foo'
+      refute_match '!', yaml
+    end
 end
