@@ -1,12 +1,9 @@
-require "test/unit"
-require "rdoc/markup/inline"
-require "rdoc/markup/to_html_crossref"
+require 'rdoc/test_case'
 
-class TestRDocMarkupAttributeManager < Test::Unit::TestCase
+class TestRDocMarkupAttributeManager < RDoc::TestCase
 
   def setup
-    @orig_special = RDoc::Markup::AttributeManager::SPECIAL
-    RDoc::Markup::AttributeManager::SPECIAL.replace Hash.new
+    super
 
     @am = RDoc::Markup::AttributeManager.new
 
@@ -29,15 +26,17 @@ class TestRDocMarkupAttributeManager < Test::Unit::TestCase
     @am.add_word_pair("{", "}", :WOMBAT)
     @wombat_on    = @am.changed_attribute_by_name([], [:WOMBAT])
     @wombat_off   = @am.changed_attribute_by_name([:WOMBAT], [])
-  end
 
-  def teardown
-    RDoc::Markup::AttributeManager::SPECIAL.replace @orig_special
+    @klass = RDoc::Markup::AttributeManager
+    @formatter = RDoc::Markup::Formatter.new @rdoc.options
+    @formatter.add_tag :BOLD, '<B>', '</B>'
+    @formatter.add_tag :EM, '<EM>', '</EM>'
+    @formatter.add_tag :TT, '<CODE>', '</CODE>'
   end
 
   def crossref(text)
-    crossref_bitmap = RDoc::Markup::Attribute.bitmap_for(:_SPECIAL_) |
-                      RDoc::Markup::Attribute.bitmap_for(:CROSSREF)
+    crossref_bitmap = @am.attributes.bitmap_for(:_SPECIAL_) |
+                      @am.attributes.bitmap_for(:CROSSREF)
 
     [ @am.changed_attribute_by_name([], [:CROSSREF, :_SPECIAL_]),
       RDoc::Markup::Special.new(crossref_bitmap, text),
@@ -51,27 +50,56 @@ class TestRDocMarkupAttributeManager < Test::Unit::TestCase
     #assert_equal(["cat {and} dog" ], @am.flow("cat \\{and} dog"))
   end
 
+  def test_add_html_tag
+    @am.add_html("Test", :TEST)
+    tags = @am.html_tags
+    assert_equal(6, tags.size)
+    assert(tags.has_key?("test"))
+  end
+
+  def test_add_special
+    @am.add_special "WikiWord", :WIKIWORD
+    specials = @am.special
+
+    assert_equal 1, specials.size
+    assert specials.assoc "WikiWord"
+  end
+
   def test_add_word_pair
     @am.add_word_pair '%', '&', 'percent and'
 
-    assert RDoc::Markup::AttributeManager::WORD_PAIR_MAP.include?(/(%)(\S+)(&)/)
-    assert RDoc::Markup::AttributeManager::PROTECTABLE.include?('%')
-    assert !RDoc::Markup::AttributeManager::PROTECTABLE.include?('&')
+    assert @am.word_pair_map.include?(/(%)(\S+)(&)/)
+    assert @am.protectable.include?('%')
+    assert !@am.protectable.include?('&')
   end
 
   def test_add_word_pair_angle
-    e = assert_raise ArgumentError do
+    e = assert_raises ArgumentError do
       @am.add_word_pair '<', '>', 'angles'
     end
 
     assert_equal "Word flags may not start with '<'", e.message
   end
 
+  def test_add_word_pair_invalid
+    assert_raises ArgumentError do
+      @am.add_word_pair("<", "<", :TEST)
+    end
+  end
+
+  def test_add_word_pair_map
+    @am.add_word_pair("x", "y", :TEST)
+
+    word_pair_map = @am.word_pair_map
+
+    assert_includes word_pair_map.keys.map { |r| r.source }, "(x)(\\S+)(y)"
+  end
+
   def test_add_word_pair_matching
     @am.add_word_pair '^', '^', 'caret'
 
-    assert RDoc::Markup::AttributeManager::MATCHING_WORD_PAIRS.include?('^')
-    assert RDoc::Markup::AttributeManager::PROTECTABLE.include?('^')
+    assert @am.matching_word_pairs.include?('^')
+    assert @am.protectable.include?('^')
   end
 
   def test_basic
@@ -123,6 +151,10 @@ class TestRDocMarkupAttributeManager < Test::Unit::TestCase
                  @am.flow("*\\bold*")
   end
 
+  def test_bold_html_escaped
+    assert_equal ['cat <b>dog</b>'], @am.flow('cat \<b>dog</b>')
+  end
+
   def test_combined
     assert_equal(["cat ", @em_on, "and", @em_off, " ", @bold_on, "dog", @bold_off],
                   @am.flow("cat _and_ *dog*"))
@@ -154,6 +186,56 @@ class TestRDocMarkupAttributeManager < Test::Unit::TestCase
     assert_equal "\000x-y\000", str
   end
 
+  def test_convert_attrs_ignores_code
+    assert_equal 'foo <CODE>__send__</CODE> bar', output('foo <code>__send__</code> bar')
+  end
+
+  def test_convert_attrs_ignores_tt
+    assert_equal 'foo <CODE>__send__</CODE> bar', output('foo <tt>__send__</tt> bar')
+  end
+
+  def test_convert_attrs_preserves_double
+    assert_equal 'foo.__send__ :bar', output('foo.__send__ :bar')
+    assert_equal 'use __FILE__ to', output('use __FILE__ to')
+  end
+
+  def test_convert_attrs_does_not_ignore_after_tt
+    assert_equal 'the <CODE>IF:</CODE><EM>key</EM> directive', output('the <tt>IF:</tt>_key_ directive')
+  end
+
+  def test_escapes
+    assert_equal '<CODE>text</CODE>',   output('<tt>text</tt>')
+    assert_equal '<tt>text</tt>',       output('\\<tt>text</tt>')
+    assert_equal '<tt>',                output('\\<tt>')
+    assert_equal '<CODE><tt></CODE>',   output('<tt>\\<tt></tt>')
+    assert_equal '<CODE>\\<tt></CODE>', output('<tt>\\\\<tt></tt>')
+    assert_equal '<B>text</B>',         output('*text*')
+    assert_equal '*text*',              output('\\*text*')
+    assert_equal '\\',                  output('\\')
+    assert_equal '\\text',              output('\\text')
+    assert_equal '\\\\text',            output('\\\\text')
+    assert_equal 'text \\ text',        output('text \\ text')
+
+    assert_equal 'and <CODE>\\s</CODE> matches space',
+                 output('and <tt>\\s</tt> matches space')
+    assert_equal 'use <CODE><tt>text</CODE></tt> for code',
+                 output('use <tt>\\<tt>text</tt></tt> for code')
+    assert_equal 'use <CODE><tt>text</tt></CODE> for code',
+                 output('use <tt>\\<tt>text\\</tt></tt> for code')
+    assert_equal 'use <tt><tt>text</tt></tt> for code',
+                 output('use \\<tt>\\<tt>text</tt></tt> for code')
+    assert_equal 'use <tt><CODE>text</CODE></tt> for code',
+                 output('use \\<tt><tt>text</tt></tt> for code')
+    assert_equal 'use <CODE>+text+</CODE> for code',
+                 output('use <tt>\\+text+</tt> for code')
+    assert_equal 'use <tt><CODE>text</CODE></tt> for code',
+                 output('use \\<tt>+text+</tt> for code')
+    assert_equal 'illegal <tag>not</tag> changed',
+                 output('illegal <tag>not</tag> changed')
+    assert_equal 'unhandled <p>tag</p> unchanged',
+                 output('unhandled <p>tag</p> unchanged')
+  end
+
   def test_html_like_em_bold
     assert_equal ["cat ", @em_on, "and ", @em_to_bold, "dog", @bold_off],
                   @am.flow("cat <i>and </i><b>dog</b>")
@@ -194,22 +276,57 @@ class TestRDocMarkupAttributeManager < Test::Unit::TestCase
                   @am.flow("<tt>cat</tt> <i>and <b></i>dog</b>")
   end
 
-  def test_protect
-    assert_equal(['cat \\ dog'], @am.flow('cat \\ dog'))
+  def test_initial_html
+    html_tags = @am.html_tags
+    assert html_tags.is_a?(Hash)
+    assert_equal(5, html_tags.size)
+  end
 
-    assert_equal(["cat <tt>dog</Tt>"], @am.flow("cat \\<tt>dog</Tt>"))
+  def test_initial_word_pairs
+    word_pairs = @am.matching_word_pairs
+    assert word_pairs.is_a?(Hash)
+    assert_equal(3, word_pairs.size)
+  end
+
+  def test_mask_protected_sequence
+    def @am.str()     @str       end
+    def @am.str=(str) @str = str end
+
+    @am.str = '<code>foo</code>'
+    @am.mask_protected_sequences
+
+    assert_equal "<code>foo</code>",       @am.str
+
+    @am.str = '<code>foo\\</code>'
+    @am.mask_protected_sequences
+
+    assert_equal "<code>foo<\x04/code>", @am.str, 'escaped close'
+
+    @am.str = '<code>foo\\\\</code>'
+    @am.mask_protected_sequences
+
+    assert_equal "<code>foo\\</code>",     @am.str, 'escaped backslash'
+  end
+
+  def test_protect
+    assert_equal(['cat \\ dog'],
+                 @am.flow('cat \\ dog'))
+
+    assert_equal(["cat <tt>dog</Tt>"],
+                 @am.flow("cat \\<tt>dog</Tt>"))
 
     assert_equal(["cat ", @em_on, "and", @em_off, " <B>dog</b>"],
                   @am.flow("cat <i>and</i> \\<B>dog</b>"))
 
-    assert_equal(["*word* or <b>text</b>"], @am.flow("\\*word* or \\<b>text</b>"))
+    assert_equal(["*word* or <b>text</b>"],
+                 @am.flow("\\*word* or \\<b>text</b>"))
 
     assert_equal(["_cat_", @em_on, "dog", @em_off],
                   @am.flow("\\_cat_<i>dog</i>"))
   end
 
   def test_special
-    @am.add_special(RDoc::Markup::ToHtmlCrossref::CROSSREF_REGEXP, :CROSSREF)
+    @am.add_special(RDoc::CrossReference::CROSSREF_REGEXP, :CROSSREF)
 
     #
     # The apostrophes in "cats'" and "dogs'" suppress the flagging of these
@@ -228,4 +345,14 @@ class TestRDocMarkupAttributeManager < Test::Unit::TestCase
     assert_equal(["cats' ", crossref("#fred")].flatten, @am.flow("cats' #fred"))
   end
 
+  def test_tt_html
+    assert_equal [@tt_on, '"\n"', @tt_off],
+                 @am.flow('<tt>"\n"</tt>')
+  end
+
+  def output str
+    @formatter.convert_flow @am.flow str
+  end
+
 end
+
