@@ -45,49 +45,6 @@ EOF
 <member>
 &a;
 </member>
-EOF
-
-    XML_WITH_NESTED_EMPTY_ENTITY = <<EOF
-<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE member [
-  <!ENTITY a "&b;&b;&b;&b;&b;&b;&b;&b;&b;&b;">
-  <!ENTITY b "&c;&c;&c;&c;&c;&c;&c;&c;&c;&c;">
-  <!ENTITY c "&d;&d;&d;&d;&d;&d;&d;&d;&d;&d;">
-  <!ENTITY d "&e;&e;&e;&e;&e;&e;&e;&e;&e;&e;">
-  <!ENTITY e "&f;&f;&f;&f;&f;&f;&f;&f;&f;&f;">
-  <!ENTITY f "&g;&g;&g;&g;&g;&g;&g;&g;&g;&g;">
-  <!ENTITY g "">
-]>
-<member>
-&a;
-</member>
-EOF
-
-    XML_WITH_NESTED_PARAMETER_ENTITY = <<EOF
-<!DOCTYPE root [
-  <!ENTITY % a "BOOM.BOOM.BOOM.BOOM.BOOM.BOOM.BOOM.BOOM.BOOM.">
-  <!ENTITY % b "%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;">
-  <!ENTITY % c "%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;">
-  <!ENTITY % d "%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;">
-  <!ENTITY % e "%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;">
-  <!ENTITY % f "%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;">
-  <!ENTITY % g "%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;">
-  <!ENTITY test "test %g;">
-]>
-<cd></cd>
-EOF
-    XML_WITH_NESTED_EMPTY_PARAMETER_ENTITY = <<EOF
-<!DOCTYPE root [
-  <!ENTITY % a "">
-  <!ENTITY % b "%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;%a;">
-  <!ENTITY % c "%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;%b;">
-  <!ENTITY % d "%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;%c;">
-  <!ENTITY % e "%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;%d;">
-  <!ENTITY % f "%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;%e;">
-  <!ENTITY % g "%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;%f;">
-  <!ENTITY test "test %g;">
-]>
-<cd></cd>
 EOF
 
   XML_WITH_4_ENTITY_EXPANSION = <<EOF
@@ -108,58 +65,24 @@ EOF
     assert_raise(RuntimeError) do
       doc.root.children.first.value
     end
-    REXML::Document.entity_expansion_limit = 100
-    assert_equal(100, REXML::Document.entity_expansion_limit)
+    REXML::Security.entity_expansion_limit = 100
+    assert_equal(100, REXML::Security.entity_expansion_limit)
     doc = REXML::Document.new(XML_WITH_NESTED_ENTITY)
     assert_raise(RuntimeError) do
       doc.root.children.first.value
     end
     assert_equal(101, doc.entity_expansion_count)
 
-    doc = REXML::Document.new(XML_WITH_NESTED_EMPTY_ENTITY)
-    assert_raise(RuntimeError) do
-      doc.root.children.first.value
-    end
-    REXML::Document.entity_expansion_limit = 100
-    assert_equal(100, REXML::Document.entity_expansion_limit)
-    doc = REXML::Document.new(XML_WITH_NESTED_EMPTY_ENTITY)
-    assert_raise(RuntimeError) do
-      doc.root.children.first.value
-    end
-    assert_equal(101, doc.entity_expansion_count)
-
-    REXML::Document.entity_expansion_limit = 4
+    REXML::Security.entity_expansion_limit = 4
     doc = REXML::Document.new(XML_WITH_4_ENTITY_EXPANSION)
     assert_equal("\na\na a\n<\n", doc.root.children.first.value)
-    REXML::Document.entity_expansion_limit = 3
+    REXML::Security.entity_expansion_limit = 3
     doc = REXML::Document.new(XML_WITH_4_ENTITY_EXPANSION)
     assert_raise(RuntimeError) do
       doc.root.children.first.value
     end
-
-    assert_raise(REXML::ParseException) do
-      REXML::Document.new(XML_WITH_NESTED_EMPTY_PARAMETER_ENTITY)
-    end
-    REXML::Document.entity_expansion_limit = 100
-    assert_equal(100, REXML::Document.entity_expansion_limit)
-    assert_raise(REXML::ParseException) do
-      REXML::Document.new(XML_WITH_NESTED_EMPTY_PARAMETER_ENTITY)
-    end
   ensure
-    REXML::Document.entity_expansion_limit = 10000
-  end
-
-  def test_entity_expansion_limit_for_parameter_entity
-    assert_raise(REXML::ParseException) do
-      REXML::Document.new(XML_WITH_NESTED_PARAMETER_ENTITY)
-    end
-    REXML::Document.entity_expansion_limit = 100
-    assert_equal(100, REXML::Document.entity_expansion_limit)
-    assert_raise(REXML::ParseException) do
-      REXML::Document.new(XML_WITH_NESTED_PARAMETER_ENTITY)
-    end
-  ensure
-    REXML::Document.entity_expansion_limit = 10000
+    REXML::Security.entity_expansion_limit = 10000
   end
 
   def test_tag_in_cdata_with_not_ascii_only_but_ascii8bit_encoding_source
