@@ -1,11 +1,15 @@
 #
 # test_scanner_events.rb
 #
+begin
+  require 'ripper'
+  require 'test/unit'
+  ripper_test = true
+  module TestRipper; end
+rescue LoadError
+end
 
-require 'ripper'
-require 'test/unit'
-
-class TestRipper_ScannerEvents < Test::Unit::TestCase
+class TestRipper::ScannerEvents < Test::Unit::TestCase
 
   def test_event_coverage
     dispatched = Ripper::SCANNER_EVENTS.map {|event,_| event }
@@ -16,7 +20,7 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
 
   def scan(target, str)
     sym = "on_#{target}".intern
-    Ripper.lex(str).select {|_,type,_| type == sym }.map {|_,_,tok| tok }
+    Ripper.lex(str).select {|_1,type,_2| type == sym }.map {|_1,_2,tok| tok }
   end
 
   def test_tokenize
@@ -28,10 +32,10 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
                  Ripper.tokenize('1')
     assert_equal ['1', ';', 'def', ' ', 'm', '(', 'arg', ')', 'end'],
                  Ripper.tokenize("1;def m(arg)end")
-    assert_equal ['print', '(', '<<EOS', ')', "\n", "heredoc\n", "EOS\n"],
-                 Ripper.tokenize("print(<<EOS)\nheredoc\nEOS\n")
-    assert_equal ['print', '(', ' ', '<<EOS', ')', "\n", "heredoc\n", "EOS\n"],
-                 Ripper.tokenize("print( <<EOS)\nheredoc\nEOS\n")
+    assert_equal ['print', '(', '<<''EOS', ')', "\n", "heredoc\n", "EOS\n"],
+                 Ripper.tokenize("print(<<""EOS)\nheredoc\nEOS\n")
+    assert_equal ['print', '(', ' ', '<<''EOS', ')', "\n", "heredoc\n", "EOS\n"],
+                 Ripper.tokenize("print( <<""EOS)\nheredoc\nEOS\n")
     assert_equal ["\#\n", "\n", "\#\n", "\n", "nil", "\n"],
                  Ripper.tokenize("\#\n\n\#\n\nnil\n")
   end
@@ -57,36 +61,68 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
                   [[2, 1], :on_nl, "\n"],
                   [[3, 0], :on_int, "3"]],
                  Ripper.lex("1\n2\n3")
-    assert_equal [[[1, 0], :on_heredoc_beg, "<<EOS"],
+    assert_equal [[[1, 0], :on_heredoc_beg, "<<""EOS"],
                   [[1, 5], :on_nl, "\n"],
                   [[2, 0], :on_tstring_content, "heredoc\n"],
                   [[3, 0], :on_heredoc_end, "EOS"]],
-                 Ripper.lex("<<EOS\nheredoc\nEOS")
+                 Ripper.lex("<<""EOS\nheredoc\nEOS")
+    assert_equal [[[1, 0], :on_heredoc_beg, "<<""EOS"],
+                  [[1, 5], :on_nl, "\n"],
+                  [[2, 0], :on_heredoc_end, "EOS"]],
+                 Ripper.lex("<<""EOS\nEOS"),
+                 "bug#4543"
+    assert_equal [[[1, 0], :on_regexp_beg, "/"],
+                  [[1, 1], :on_tstring_content, "foo\nbar"],
+                  [[2, 3], :on_regexp_end, "/"]],
+                 Ripper.lex("/foo\nbar/")
+    assert_equal [[[1, 0], :on_regexp_beg, "/"],
+                  [[1, 1], :on_tstring_content, "foo\n\u3020"],
+                  [[2, 3], :on_regexp_end, "/"]],
+                 Ripper.lex("/foo\n\u3020/")
+    assert_equal [[[1, 0], :on_tstring_beg, "'"],
+                  [[1, 1], :on_tstring_content, "foo\n\xe3\x80\xa0"],
+                  [[2, 3], :on_tstring_end, "'"]],
+                 Ripper.lex("'foo\n\xe3\x80\xa0'")
+    assert_equal [[[1, 0], :on_tstring_beg, "'"],
+                  [[1, 1], :on_tstring_content, "\u3042\n\u3044"],
+                  [[2, 3], :on_tstring_end, "'"]],
+                 Ripper.lex("'\u3042\n\u3044'")
+    assert_equal [[[1, 0], :on_rational, "1r"],
+                  [[1, 2], :on_nl, "\n"],
+                  [[2, 0], :on_imaginary, "2i"],
+                  [[2, 2], :on_nl, "\n"],
+                  [[3, 0], :on_imaginary, "3ri"],
+                  [[3, 3], :on_nl, "\n"],
+                  [[4, 0], :on_rational, "4.2r"],
+                  [[4, 4], :on_nl, "\n"],
+                  [[5, 0], :on_imaginary, "5.6ri"],
+                 ],
+                 Ripper.lex("1r\n2i\n3ri\n4.2r\n5.6ri")
   end
 
   def test_location
-    validate_location ""
-    validate_location " "
-    validate_location "@"
-    validate_location "\n"
-    validate_location "\r\n"
-    validate_location "\n\n\n\n\n\r\n\n\n"
-    validate_location "\n;\n;\n;\n;\n"
-    validate_location "nil"
-    validate_location "@ivar"
-    validate_location "1;2;3"
-    validate_location "1\n2\n3"
-    validate_location "1\n2\n3\n"
-    validate_location "def m(a) nil end"
-    validate_location "if true then false else nil end"
-    validate_location "BEGIN{print nil}"
-    validate_location "%w(a b\nc\r\nd \ne )"
-    validate_location %Q["a\nb\r\nc"]
-    validate_location "print(<<EOS)\nheredoc\nEOS\n"
-    validate_location %Q[print(<<-"EOS")\nheredoc\n     EOS\n]
+    assert_location ""
+    assert_location " "
+    assert_location ":"
+    assert_location "\n"
+    assert_location "\r\n"
+    assert_location "\n\n\n\n\n\r\n\n\n"
+    assert_location "\n;\n;\n;\n;\n"
+    assert_location "nil"
+    assert_location "@ivar"
+    assert_location "1;2;3"
+    assert_location "1\n2\n3"
+    assert_location "1\n2\n3\n"
+    assert_location "def m(a) nil end"
+    assert_location "if true then false else nil end"
+    assert_location "BEGIN{print nil}"
+    assert_location "%w(a b\nc\r\nd \ne )"
+    assert_location %Q["a\nb\r\nc"]
+    assert_location "print(<<""EOS)\nheredoc\nEOS\n"
+    assert_location "print(<<-\"EOS\")\nheredoc\n     EOS\n"
   end
 
-  def validate_location(src)
+  def assert_location(src)
     buf = ''
     Ripper.lex(src).each do |pos, type, tok|
       line, col = *pos
@@ -115,7 +151,7 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
     assert_equal [],
                  scan('comma', %q[".,.,.,.,.,.,.."])
     assert_equal [],
-                 scan('comma', %Q[<<EOS\n,,,,,,,,,,\nEOS])
+                 scan('comma', "<<""EOS\n,,,,,,,,,,\nEOS")
   end
 
   def test_period
@@ -181,12 +217,10 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
     assert_equal ['#{'],
                  scan('embexpr_beg', '%Q[#{expr}]')
     assert_equal ['#{'],
-                 scan('embexpr_beg', "m(<<EOS)\n\#{expr}\nEOS")
+                 scan('embexpr_beg', "m(<<""EOS)\n\#{expr}\nEOS")
   end
 
   def test_embexpr_end
-=begin
-    # currently detected as "rbrace"
     assert_equal [],
                  scan('embexpr_end', '')
     assert_equal ['}'],
@@ -196,8 +230,7 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
     assert_equal ['}'],
                  scan('embexpr_end', '%Q[#{expr}]')
     assert_equal ['}'],
-                 scan('embexpr_end', "m(<<EOS)\n\#{expr}\nEOS")
-=end
+                 scan('embexpr_end', "m(<<""EOS)\n\#{expr}\nEOS")
   end
 
   def test_embvar
@@ -242,6 +275,13 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
                  scan('float', 'm(a,b,1.0,c,d)')
   end
 
+  def test_rational
+    assert_equal [],
+                 scan('rational', '')
+    assert_equal ['1r', '10r', '10.1r'],
+                 scan('rational', 'm(1r,10r,10.1r)')
+  end
+
   def test_gvar
     assert_equal [],
                  scan('gvar', '')
@@ -264,6 +304,13 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
                  scan('ident', 'm(lvar, @ivar, @@cvar, $gvar)')
   end
 
+  def test_imaginary
+    assert_equal [],
+                 scan('imaginary', '')
+    assert_equal ['1i', '10ri', '10.0i', '10.1ri'],
+                 scan('imaginary', 'm(1i,10ri,10.0i,10.1ri)')
+  end
+
   def test_int
     assert_equal [],
                  scan('int', '')
@@ -458,8 +505,8 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
                  scan('op', '1 < 1')
     assert_equal ['<='],
                  scan('op', '1 <= 1')
-    assert_equal ['<<'],
-                 scan('op', '1 << 1')
+    assert_equal ['<''<'],
+                 scan('op', '1 <''< 1')
     assert_equal ['>>'],
                  scan('op', '1 >> 1')
     assert_equal ['+'],
@@ -525,6 +572,12 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
                  scan('tstring_content', '"abc#{1}def"')
     assert_equal ['sym'],
                  scan('tstring_content', ':"sym"')
+    assert_equal ['a b c'],
+                 scan('tstring_content', ':"a b c"'),
+                 "bug#4544"
+    assert_equal ["a\nb\nc"],
+                 scan('tstring_content', ":'a\nb\nc'"),
+                 "bug#4544"
   end
 
   def test_tstring_end
@@ -580,6 +633,28 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
                  scan('qwords_beg', '%w( w w w )')
   end
 
+  def test_qsymbols_beg
+    assert_equal [],
+                 scan('qsymbols_beg', '')
+    assert_equal ['%i('],
+                 scan('qsymbols_beg', '%i()')
+    assert_equal ['%i('],
+                 scan('qsymbols_beg', '%i(w w w)')
+    assert_equal ['%i( '],
+                 scan('qsymbols_beg', '%i( w w w )')
+  end
+
+  def test_symbols_beg
+    assert_equal [],
+                 scan('symbols_beg', '')
+    assert_equal ['%I('],
+                 scan('symbols_beg', '%I()')
+    assert_equal ['%I('],
+                 scan('symbols_beg', '%I(w w w)')
+    assert_equal ['%I( '],
+                 scan('symbols_beg', '%I( w w w )')
+  end
+
   # FIXME: Close paren must not present (`words_end' scanner event?).
   def test_words_sep
     assert_equal [],
@@ -597,48 +672,58 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
   def test_heredoc_beg
     assert_equal [],
                  scan('heredoc_beg', '')
-    assert_equal ['<<EOS'],
-                 scan('heredoc_beg', "<<EOS\nheredoc\nEOS")
-    assert_equal ['<<EOS'],
-                 scan('heredoc_beg', "<<EOS\nheredoc\nEOS\n")
-    assert_equal ['<<EOS'],
-                 scan('heredoc_beg', "<<EOS\nheredoc\nEOS \n")
-    assert_equal ['<<-EOS'],
-                 scan('heredoc_beg', "<<-EOS\nheredoc\n\tEOS \n")
-    assert_equal ['<<"EOS"'],
-                 scan('heredoc_beg', %Q[<<"EOS"\nheredoc\nEOS])
-    assert_equal [%q(<<'EOS')],
-                 scan('heredoc_beg', "<<'EOS'\nheredoc\nEOS")
-    assert_equal [%q(<<`EOS`)],
-                 scan('heredoc_beg', "<<`EOS`\nheredoc\nEOS")
-    assert_equal [%q(<<" ")],
-                 scan('heredoc_beg', %Q[<<" "\nheredoc\nEOS])
+    assert_equal ['<<''EOS'],
+                 scan('heredoc_beg', "<<""EOS\nheredoc\nEOS")
+    assert_equal ['<<''EOS'],
+                 scan('heredoc_beg', "<<""EOS\nheredoc\nEOS\n")
+    assert_equal ['<<''EOS'],
+                 scan('heredoc_beg', "<<""EOS\nheredoc\nEOS \n")
+    assert_equal ['<<''-EOS'],
+                 scan('heredoc_beg', "<<""-EOS\nheredoc\n\tEOS \n")
+    assert_equal ['<<''"EOS"'],
+                 scan('heredoc_beg', '<<''"EOS"'"\nheredoc\nEOS")
+    assert_equal ["<<""'EOS'"],
+                 scan('heredoc_beg', "<<""'EOS'\nheredoc\nEOS")
+    assert_equal ['<<''`EOS`'],
+                 scan('heredoc_beg', "<<""`EOS`\nheredoc\nEOS")
+    assert_equal ['<<''" "'],
+                 scan('heredoc_beg', '<<''" "'"\nheredoc\nEOS")
   end
 
   def test_tstring_content_HEREDOC
     assert_equal [],
                  scan('tstring_content', '')
     assert_equal ["heredoc\n"],
-                 scan('tstring_content', "<<EOS\nheredoc\nEOS")
+                 scan('tstring_content', "<<""EOS\nheredoc\nEOS")
     assert_equal ["heredoc\n"],
-                 scan('tstring_content', "<<EOS\nheredoc\nEOS\n")
-    assert_equal ["heredoc \n"],
-                 scan('tstring_content', "<<EOS\nheredoc \nEOS \n")
-    assert_equal ["heredoc\n"],
-                 scan('tstring_content', "<<-EOS\nheredoc\n\tEOS \n")
+                 scan('tstring_content', "<<""EOS\nheredoc\nEOS\n")
+    assert_equal ["here\ndoc \nEOS \n"],
+                 scan('tstring_content', "<<""EOS\nhere\ndoc \nEOS \n")
+    assert_equal ["heredoc\n\tEOS \n"],
+                 scan('tstring_content', "<<""-EOS\nheredoc\n\tEOS \n")
+    bug7255 = '[ruby-core:48703]'
+    assert_equal ["there\n""heredoc", "\n"],
+                 scan('tstring_content', "<<""EOS\n""there\n""heredoc\#{foo}\nEOS"),
+                 bug7255
+    assert_equal ["there\n""heredoc", "\n"],
+                 scan('tstring_content', "<<""EOS\n""there\n""heredoc\#@foo\nEOS"),
+                 bug7255
   end
 
   def test_heredoc_end
     assert_equal [],
                  scan('heredoc_end', '')
     assert_equal ["EOS"],
-                 scan('heredoc_end', "<<EOS\nheredoc\nEOS")
+                 scan('heredoc_end', "<<""EOS\nEOS"),
+                 "bug#4543"
+    assert_equal ["EOS"],
+                 scan('heredoc_end', "<<""EOS\nheredoc\nEOS")
     assert_equal ["EOS\n"],
-                 scan('heredoc_end', "<<EOS\nheredoc\nEOS\n")
-    assert_equal ["EOS \n"],
-                 scan('heredoc_end', "<<EOS\nheredoc\nEOS \n")
-    assert_equal ["\tEOS \n"],
-                 scan('heredoc_end', "<<-EOS\nheredoc\n\tEOS \n")
+                 scan('heredoc_end', "<<""EOS\nheredoc\nEOS\n")
+    assert_equal [],
+                 scan('heredoc_end', "<<""EOS\nheredoc\nEOS \n")
+    assert_equal [],
+                 scan('heredoc_end', "<<""-EOS\nheredoc\n\tEOS \n")
   end
 
   def test_semicolon
@@ -782,8 +867,8 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
   def test_CHAR
     assert_equal [],
                  scan('CHAR', "")
-    assert_equal ["@"],
-                 scan('CHAR', "@")
+    assert_equal ["?a"],
+                 scan('CHAR', "?a")
     assert_equal [],
                  scan('CHAR', "@ivar")
   end
@@ -800,4 +885,4 @@ class TestRipper_ScannerEvents < Test::Unit::TestCase
   def test_tlambda_arg
   end
 
-end
+end if ripper_test
