@@ -15,6 +15,15 @@ class TestTime < Test::Unit::TestCase
     $VERBOSE = @verbose
   end
 
+  def in_timezone(zone)
+    orig_zone = ENV['TZ']
+
+    ENV['TZ'] = zone
+    yield
+  ensure
+    ENV['TZ'] = orig_zone
+  end
+
   def no_leap_seconds?
     # 1972-06-30T23:59:60Z is the first leap second.
     Time.utc(1972, 7, 1, 0, 0, 0) - Time.utc(1972, 6, 30, 23, 59, 59) == 1
@@ -47,14 +56,14 @@ class TestTime < Test::Unit::TestCase
                  Time.utc(2000, 3, 21, 0, 30))
     assert_equal(0, (Time.at(1.1) + 0.9).usec)
 
-    assert((Time.utc(2000, 4, 1) + 24).utc?)
-    assert(!(Time.local(2000, 4, 1) + 24).utc?)
+    assert_predicate((Time.utc(2000, 4, 1) + 24), :utc?)
+    assert_not_predicate((Time.local(2000, 4, 1) + 24), :utc?)
 
     t = Time.new(2000, 4, 1, 0, 0, 0, "+01:00") + 24
-    assert(!t.utc?)
+    assert_not_predicate(t, :utc?)
     assert_equal(3600, t.utc_offset)
     t = Time.new(2000, 4, 1, 0, 0, 0, "+02:00") + 24
-    assert(!t.utc?)
+    assert_not_predicate(t, :utc?)
     assert_equal(7200, t.utc_offset)
   end
 
@@ -291,18 +300,15 @@ class TestTime < Test::Unit::TestCase
   end
 
   def test_marshal_zone
-    orig_zone = ENV['TZ']
-
     t = Time.utc(2013, 2, 24)
     assert_equal('UTC', t.zone)
     assert_equal('UTC', Marshal.load(Marshal.dump(t)).zone)
 
-    ENV['TZ'] = 'JST-9'
-    t = Time.local(2013, 2, 24)
-    assert_equal('JST', Time.local(2013, 2, 24).zone)
-    assert_equal('JST', Marshal.load(Marshal.dump(t)).zone)
-  ensure
-    ENV['TZ'] = orig_zone
+    in_timezone('JST-9') do
+      t = Time.local(2013, 2, 24)
+      assert_equal('JST', Time.local(2013, 2, 24).zone)
+      assert_equal('JST', Marshal.load(Marshal.dump(t)).zone)
+    end
   end
 
   def test_marshal_to_s
@@ -312,13 +318,31 @@ class TestTime < Test::Unit::TestCase
       "[ruby-dev:44827] [Bug #5586]")
   end
 
-  def test_security_error
-    assert_raise(SecurityError) do
-      Thread.new do
-        t = Time.gm(2000)
-        $SAFE = 4
-        t.localtime
-      end.join
+  Bug8795 = '[ruby-core:56648] [Bug #8795]'
+
+  def test_marshal_broken_offset
+    data = "\x04\bIu:\tTime\r\xEFF\x1C\x80\x00\x00\x00\x00\x06:\voffset"
+    t1 = t2 = nil
+    in_timezone('UTC') do
+      assert_nothing_raised(TypeError, ArgumentError, Bug8795) do
+        t1 = Marshal.load(data + "T")
+        t2 = Marshal.load(data + "\"\x0ebadoffset")
+      end
+      assert_equal(0, t1.utc_offset)
+      assert_equal(0, t2.utc_offset)
+    end
+  end
+
+  def test_marshal_broken_zone
+    data = "\x04\bIu:\tTime\r\xEFF\x1C\x80\x00\x00\x00\x00\x06:\tzone"
+    t1 = t2 = nil
+    in_timezone('UTC') do
+      assert_nothing_raised(TypeError, ArgumentError, Bug8795) do
+        t1 = Marshal.load(data + "T")
+        t2 = Marshal.load(data + "\"\b\0\0\0")
+      end
+      assert_equal('UTC', t1.zone)
+      assert_equal('UTC', t2.zone)
     end
   end
 
@@ -394,14 +418,14 @@ class TestTime < Test::Unit::TestCase
 
   def test_eql
     t2000 = get_t2000
-    assert(t2000.eql?(t2000))
-    assert(!t2000.eql?(Time.gm(2001)))
+    assert_operator(t2000, :eql?, t2000)
+    assert_not_operator(t2000, :eql?, Time.gm(2001))
   end
 
   def test_utc_p
-    assert(Time.gm(2000).gmt?)
-    assert(!Time.local(2000).gmt?)
-    assert(!Time.at(0).gmt?)
+    assert_predicate(Time.gm(2000), :gmt?)
+    assert_not_predicate(Time.local(2000), :gmt?)
+    assert_not_predicate(Time.at(0), :gmt?)
   end
 
   def test_hash
@@ -429,15 +453,15 @@ class TestTime < Test::Unit::TestCase
   def test_localtime_gmtime
     assert_nothing_raised do
       t = Time.gm(2000)
-      assert(t.gmt?)
+      assert_predicate(t, :gmt?)
       t.localtime
-      assert(!t.gmt?)
+      assert_not_predicate(t, :gmt?)
       t.localtime
-      assert(!t.gmt?)
+      assert_not_predicate(t, :gmt?)
       t.gmtime
-      assert(t.gmt?)
+      assert_predicate(t, :gmt?)
       t.gmtime
-      assert(t.gmt?)
+      assert_predicate(t, :gmt?)
     end
 
     t1 = Time.gm(2000)
@@ -530,13 +554,13 @@ class TestTime < Test::Unit::TestCase
     assert_equal("UTC", t2000.zone)
     assert_equal(Encoding.find("locale"), t2000.zone.encoding)
     assert_equal(0, t2000.gmt_offset)
-    assert(!t2000.sunday?)
-    assert(!t2000.monday?)
-    assert(!t2000.tuesday?)
-    assert(!t2000.wednesday?)
-    assert(!t2000.thursday?)
-    assert(!t2000.friday?)
-    assert(t2000.saturday?)
+    assert_not_predicate(t2000, :sunday?)
+    assert_not_predicate(t2000, :monday?)
+    assert_not_predicate(t2000, :tuesday?)
+    assert_not_predicate(t2000, :wednesday?)
+    assert_not_predicate(t2000, :thursday?)
+    assert_not_predicate(t2000, :friday?)
+    assert_predicate(t2000, :saturday?)
     assert_equal([0, 0, 0, 1, 1, 2000, 6, 1, false, "UTC"], t2000.to_a)
 
     t = Time.at(946684800).getlocal
@@ -884,20 +908,6 @@ class TestTime < Test::Unit::TestCase
     assert_raise(NoMethodError, bug5012) { t1.m }
   end
 
-  def test_time_subclass
-    bug5036 = '[ruby-dev:44122]'
-    tc = Class.new(Time)
-    tc.inspect
-    t = tc.now
-    error = assert_raise(SecurityError) do
-      proc do
-        $SAFE = 4
-        t.gmtime
-      end.call
-    end
-    assert_equal("Insecure: can't modify #{tc}", error.message, bug5036)
-  end
-
   def test_sec_str
     bug6193 = '[ruby-core:43569]'
     t = nil
